  # R script to evaluate single link SimGrid model
# Author        : Pedro Velho
# Last Modified : 04/11/2008  
gnuplot3D = function (X,Y,Z,minX=0,maxX=5,minY=0,maxY=5,minZ=0,maxZ=5,labelX="X",labelY="Y",labelZ="Z", ticsX="1", ticsY="1", ticsZ="1", title="Blah", exportSvg = FALSE, outputfile="./tmp/gnuplot3d.svg") {
  #export data to gnuplot file
  outFile = file("./tmp/gnuplot3d.data", "w")
  cat("#",labelX,labelY,labelZ,"\n",sep=" ")
  write.table(data.frame(X,Y,Z), file = outFile , row.names = FALSE, col.names = FALSE)
  close(outFile)
   
  #export gnuplot script
  outFile = file("./tmp/gnuplot3d.script", "w")
  cat(file = outFile, sep = "", "set xrange [",minX, ":",maxX,"]\n")
  cat(file = outFile, sep = "", "set yrange [",minY, ":",maxY,"]\n")
  cat(file = outFile, sep = "", "set zrange [",minZ, ":",maxZ,"]\n")
  cat(file = outFile, sep="", "set xlabel \"",labelX,"\"\n")
  cat(file = outFile, sep="", "set ylabel \"",labelY,"\"\n")
  cat(file = outFile, sep="", "set zlabel \"",labelZ,"\"\n")  
  cat(file = outFile, sep="", "set xtics ",ticsX,"\n")
  cat(file = outFile, sep="", "set ytics ",ticsY,"\n")
  cat(file = outFile, sep="", "set ztics ",ticsZ,"\n")
  cat(file = outFile, sep="", "set key bottom right\n")
  cat(file = outFile, sep="", "set title \"", title, "\"\n")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set term svg \n")
    cat(file = outFile, sep = "", "set output '",outputfile,"'\n")
  }

  cat(file = outFile, "splot x*0.5 + y , './tmp/gnuplot3d.data' ls 2 lt rgb \"#093267\" lw 1 t 'GTNetS' \n")

  close(outFile)

  #call gnuplot and present 3D data
  system("gnuplot -persist ./tmp/gnuplot3d.script")
}



gnuplotSimpleModel = function (X, Y, minX=0, maxX=5, minY=0, maxY=5, labelX="X",labelY="Y", ticsX="1", ticsY="1", title="Blah", exportSvg = FALSE, outputfile="./tmp/gnuplotSimpleModel.svg") {
  
                                        #export data to gnuplot file
  outFile = file("./tmp/gnuplot2d.data", "w")
  cat("#",labelX,labelY,"\n",sep=" ")
  write.table(data.frame(X,Y), file = outFile , row.names = FALSE, col.names = FALSE)
  close(outFile)
   
  #export gnuplot script
  outFile = file("./tmp/gnuplot2d.script", "w")
  cat(file = outFile, sep = "", "set xrange [",minX, ":",maxX,"]\n")
  cat(file = outFile, sep = "", "set yrange [",minY, ":",maxY,"]\n")
  cat(file = outFile, sep="", "set xlabel \"",labelX,"\"\n")
  cat(file = outFile, sep="", "set ylabel \"",labelY,"\"\n")
  cat(file = outFile, sep="", "set xtics ",ticsX,"\n")
  cat(file = outFile, sep="", "set ytics ",ticsY,"\n")
  cat(file = outFile, sep = "", "unset key\n")
  cat(file = outFile, sep="", "set title \"", title, "\"\n")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set term svg\n")
    cat(file = outFile, sep = "", "set output '",outputfile,"'\n")
  }
  
  cat(file = outFile, "plot './tmp/gnuplot2d.data' ls 1 lt rgb \"#ACACAC\"  t 'GTNetS', (0.5)*x ls 1 lt rgb \"#000000\" lw 2 t 'f(W) = W/2' \n")
  close(outFile)

  #call gnuplot and present 2D data
  system("gnuplot -persist ./tmp/gnuplot2d.script")
}

#############################################################################
# ./figures/ch6-02.pdf
#############################################################################
auxTable = read.table("./dat/raw-1-130.data")
gnuplotSimpleModel(auxTable$Window, (auxTable$Size*auxTable$Latency)/auxTable$Time, minX=0 , maxX=131072, minY=450, maxY=32000, labelX="W (TCP maximum window)", labelY="(LS)/T", ticsX="(20000, 40000, 65535, 80000, 131071)" , ticsY="5000", title="f(W) = (LS)/T", exportSvg=FALSE, outputfile="./tmp/throughput-vs-wmax.svg")
#V. Jacobson, R. Braden, and D. Borman, “TCP extensions for high performance,”
#May 1992. RFC 1323.


#############################################################################
# Plot a 3D graph with f(W)=(1e7L)/T , S=10 MBgnuplot svg options
#############################################################################
auxTable = read.table("./dat/raw-1-130.data")
gnuplot3D(auxTable$Window, auxTable$Latency, (auxTable$Size*auxTable$Latency)/auxTable$Time,  minX=5000 , maxX=65535, minY=0.010, maxY=0.510, minZ=450, maxZ=35000, labelX="TCP maximum window", labelY="latency", labelZ="(1e7 * L) / T", ticsX="(20000, 40000, 65535, 80000, 131071)" , ticsY="0.1", ticsZ="5000", title="f(w) = (LS)/T , for S = 10 MB (1e7)", exportSvg=FALSE, outputfile="./tmp/result-varying-latency.svg")





#############################################################################
# Execute a linear regression to verify if (1/2)*W is a good approximation
#############################################################################
auxTable = read.table("./dat/raw-1-130.data")
auxTable = auxTable[(auxTable$Window <= 65535),]
y = (auxTable$Size*auxTable$Latency)/auxTable$Time
x = auxTable$Window
# linear regression with no intercept force curve
# to pass through (0,0) (Y ~ 1 - X)
reg = lm(y ~ -1 + x)
# 0.4928667 it is in fact better but 1/2=0.5 is good too
summary(reg)






gnuplotRegModel = function (X, Y, minX=0, maxX=5, minY=0, maxY=5, labelX="X",labelY="Y", ticsX="1", ticsY="1", title="Blah", exportSvg = FALSE) {

                                        #export data to gnuplot file
  outFile = file("./tmp/gnuplot2d.data", "w")
  cat("#",labelX,labelY,"\n",sep=" ")
  write.table(data.frame(X,Y), file = outFile , row.names = FALSE, col.names = FALSE)
  close(outFile)
   
  #export gnuplot script
  outFile = file("./tmp/gnuplot2d.script", "w")
  cat(file = outFile, sep = "", "set xrange [",minX, ":",maxX,"]\n")
  cat(file = outFile, sep = "", "set yrange [",minY, ":",maxY,"]\n")
  cat(file = outFile, sep="", "set xlabel \"",labelX,"\"\n")
  cat(file = outFile, sep="", "set ylabel \"",labelY,"\"\n")
  cat(file = outFile, sep="", "set xtics ",ticsX,"\n")
  cat(file = outFile, sep="", "set ytics ",ticsY,"\n")
  cat(file = outFile, sep="", "unset key\n")
  cat(file = outFile, sep="", "set title \"", title, "\"\n")

  if(exportSvg) {
    cat(file = outFile, sep = "", "set term svg enhanced\n")
    cat(file = outFile, sep = "", "set output './tmp/gnuplot3Dbw.svg'\n")
  }
  
  cat(file = outFile, "plot './tmp/gnuplot2d.data' ls 7 lw 0.2 t 'GTNetS', (0.4928667)*x ls 1 lw 2 t 'f(W) = 0.4928667W'\n")
  close(outFile)

  #call gnuplot and present 2D data
  system("gnuplot -persist ./tmp/gnuplot2d.script")
}


#############################################################################
# Plot 2D graph with linear regression model 0.4928667*W
#############################################################################
auxTable = read.table("./dat/raw-1-130.data")
auxTable = auxTable[(auxTable$Window <= 65535),]
gnuplotRegModel(auxTable$Window, (auxTable$Size*auxTable$Latency)/auxTable$Time, minX=10000 , maxX=65536, minY=450, maxY=32000, labelX="TCP maximum window", labelY="(1e7 * 0.5) / T", ticsX="(10000, 20000, 30000, 40000, 50000, 65535)" , ticsY="5000", title="f(w) = (LS)/T", exportSvg=FALSE)

# compute error of 0.5*W
error = abs(log((auxTable$Size*auxTable$Latency)/auxTable$Time) - log(auxTable$Window/2))
max(error)
min(error)

# compute error of 0.4928667*W
error = abs(log((auxTable$Size*auxTable$Latency)/auxTable$Time) - log(auxTable$Window*0.4928667))
max(error)
min(error)

