#!/usr/bin/perl -w
# Author        : Pedro Velho
# last modified : 27/05/2010


$workingdir="/home/velho/usr/simgrid-3.4.1/examples/msg/gtnets";
if ( -d $workingdir ){
    $thisdir=`pwd`;   
    chomp($thisdir);
}else{
    print "Local working directory doesn't exist!\n";
    print "Trying grid5000...\n";

    $workingdir="/home/grenoble/pvelho/usr/simgrid-3.4.1/examples/msg/gtnets";
    
    if ( -d $workingdir ){
	$thisdir=`pwd`;    
	chomp($thisdir);
    }else{
	print "Could not find working directory. Aborting...\n";
	exit 1;
    }
}


# ****** 1- Parameter Message Size (S) ******
# in BYTES
@message_size_list = ("10000000", "20000000", "30000000", "40000000", "50000000");

# ****** 2- Parameter Latency (L) ****** 
# in SECONDS
@latency_list=("0.10000", "0.20000", "0.30000", "0.40000", "0.50000");

# ****** 3- Bandwidth (B) ******
# in BYTESperSECOND
@bandwidth_list=("125000000");

# ****** 4- Model (M) ******
@model_list=("GTNets");



# ****** 5- Maximum Window (Wmax) ******
@window_list=(  "500", "1000",  "2000",  "3000",  "4000",  "5000",  "6000",  "7000",  "8000",  "9000", "10000", "11000", "12000", "13000", "14000", "15000", "16000", "17000", "18000", "19000", "20000",  "21000",  "22000",  "23000",  "24000",  "25000",  "26000",  "27000",  "28000",  "29000", "30000",  "31000",  "32000",  "33000",  "34000",  "35000",  "36000",  "37000",  "38000",  "39000", "40000",  "41000",  "42000",  "43000",  "44000",  "45000",  "46000",  "47000",  "48000",  "49000", "50000",  "51000",  "52000",  "53000",  "54000",  "55000",  "56000",  "57000",  "58000",  "59000", "60000",  "61000",  "62000",  "63000",  "64000",  "65000", "65535",  "67000",  "68000",  "69000", "70000",  "71000",  "72000",  "73000",  "74000",  "75000",  "76000",  "77000",  "78000",  "79000", "80000",  "81000",  "82000",  "83000",  "84000",  "85000",  "86000",  "87000",  "88000",  "89000", "90000",  "91000",  "92000",  "93000",  "94000",  "95000",  "96000",  "97000",  "98000",  "99000", "100000", "101000", "102000", "103000", "104000", "105000", "106000", "107000", "108000", "109000", "110000", "111000", "112000", "113000", "114000", "115000", "116000", "117000", "118000", "119000", "120000", "121000", "122000", "123000", "124000", "125000", "126000", "127000", "128000", "131071");

#print parameter range
my($inter, $avg, $max, $min, $count);
$avg=0;
$max = 0;
$min = 1000000;
$count = 1;
foreach $inter (@window_list)
{
    if($inter > $max){
	$max = $inter;
    }
    if($inter < $min){
	$min = $inter;
    }
    $avg+=$inter;
    $count++;
}
print "TCP_wmax with ".$count." points in [".$min." B ; ".$max." B] AVG ".$avg/$count."\n";
print "B (Bandwidth) is ".$bandwidth_list[0]."\n";
print "L (Latency)   is ".$latency_list[0]."\n";
print "S (Size)      is ".$message_size_list[0]."\n";






#########################################
# Parse and verify arguments range
#########################################
sub parseParameters() {
    #adjust to perl parameters (arrays are from 0 up size-1)
    $task_begin=$ARGV[1]-1;
    $task_end=$ARGV[2]-1;

    if($task_begin < 0){
	print "Starting from a not valuable array index aborting...\n";
	print "Check the correctness of first parameter.\n";
	exit 1;
    }
    print "Bandiwthd array size is : ",scalar(@window_list),"\n";
    if($task_end >= scalar(@window_list)){
	print "Error doing more tasks than we have to do aborting...\n";
	print "Check the correctness of second parameter.\n";
	exit 1;
    }
    if($task_begin > $task_end){
	print "Error starting point is greater then end point...\n";
	print "Second parameter must be greater.\n";
	exit 1;
    }
}




#########################################
# Print error and exit
#########################################
sub printError {
    my($errorMsg) = $_[0];
    print "Error : ".$errorMsg."\n";
    print "Parse usage:\n";
    print "\tsweep_analyze parse <task_begin> <task_end>\n\n";
    print "Sweep usage:\n";
    print "\tsweep_analyze sweep <task_begin> <task_end>\n\n";
    print "Examples\n";
    print "\tsweep_analyze sweep 1 1 # executes task 1 (fix bandwidth[0] parameter)\n";
    print "\tsweep_analyze sweep 1 3 # executes task from 1 up to 3\n\n";
    print "\tsweep_analyze parse 1 3 # parse trace files of tasks 1, 2 and 3\n\n";

    print "\nIn this program you have ".($#window_list+1)." tasks to parallelize.\n\n";
    exit 1;
}

if(scalar(@ARGV) < 1){ 
    printError("Needs at least one parameter sweep or parse");
}

if($ARGV[0] ne "sweep" && $ARGV[0] ne "parse"){
    printError("First argument should be sweep or parse");
}

if($ARGV[0] eq "sweep"){
    if(scalar(@ARGV) != 3){
	printError("Option sweep must be followed by two integers in task range.");
    }else {
	executeApplication();
    }
}

if($ARGV[0] eq "parse"){
    if(scalar(@ARGV) != 3){
	printError("Option parse must be followed by two integers in task range.");
    }else {	
	parseParameters();
	parseResults(($task_begin+1), ($task_end+1));
    }
}




#########################################
# Execute application only once
#########################################
sub executeApplication {
    parseParameters();
    
    print "Changing working directory to ".$workingdir."\n";
    chdir($workingdir);

    $tmpoutputxmlpla = $thisdir."/tmp/plateforme-".($task_begin+1)."-".($task_end+1).".xml";
    $tmpoutputxmldep = $thisdir."/tmp/deployment-".($task_begin+1)."-".($task_end+1).".xml";
    $tmpoutputlog    = $thisdir."/tmp/tempotrace-".($task_begin+1)."-".($task_end+1).".log";

    open(LOGOUTPUT, ">" , $thisdir."/log/trace-file-".($task_begin+1)."-".($task_end+1).".log") ||
	die "Sorry, couldn't open file ".$thisdir."/log/trace-file-".($task_begin+1)."-".($task_end+1).".log for writting...\n";

    print LOGOUTPUT "\n";

    $it=0;
    for($it=$task_begin; $it <= $task_end; $it++) 
    {
	$TCP_window = $window_list[$it];
	print ">=============================================================<\n";
	print "========> Window (W) : ",$TCP_window," B/s (Bytes per second)\n";

	foreach $bandwidth (@bandwidth_list)
	{
	    print "========> Bandwidth (B) : ",$bandwidth," B/s\n";


	    foreach $latency (@latency_list)
	    {
		print "========> Latency   (L) : ",$latency," s (seconds)\n";

		foreach $size (@message_size_list)
		{
		    print "========> Size      (S) : ",$size," B (Bytes) \n";		
		    foreach $model (@model_list)
		    {
			print "========> Model     (M) : ",$model,"\n";

			# Log some stuff into the file to know used parameters
			print LOGOUTPUT ">==================================================<\n";
			print LOGOUTPUT "========> Bandwidth (B) : ",$bandwidth," B/s (Bytes per second)\n";
			print LOGOUTPUT "========> Latency   (L) : ",$latency," s (seconds)\n";
			print LOGOUTPUT "========> Size      (S) : ",$size," B (Bytes) \n";
			print LOGOUTPUT "========> Window    (W) : ",$TCP_window," B (Bytes) \n";
			print LOGOUTPUT "========> Model     (M) : ",$model,"\n";
		    
		    
			# Set parameters
			system ("sed -e s/bw/".$bandwidth."/g -e s/lt/".$latency."/g ".$thisdir."/onelink-p-template.xml > ".$tmpoutputxmlpla);
			system ("sed -e s/size/".$size."/g ".$thisdir."/onelink-d-template.xml > ".$tmpoutputxmldep);
		    
			# Run with parameters
			system($workingdir."/gtnets ".$tmpoutputxmlpla." ".$tmpoutputxmldep." --cfg=workstation/model:compound --cfg=cpu/model:Cas01 --cfg=network/model:".$model."  --cfg=fullduplex:0 --cfg=TCP_gamma:".$TCP_window." 1>&2 2> ".$tmpoutputlog."\n");
		    
			# Open temporary output file
			open(TMPOUTPUT, "<".$tmpoutputlog) || 
			die "Sorry, could't open temporary result file...\n";
		    
			# Copy execution trace into global trace file
			while(<TMPOUTPUT>)
			{
			    my $line = $_;
			    chomp $line;
			    print LOGOUTPUT $line."\n";
			}
			print LOGOUTPUT "=========================><=========================\n";
			close(TMPOUTPUT);
		    }
		}
	    }
	}
	print "==============================><=============================\n";
    }
    close(LOGOUTPUT);
}



#########################################
# Parse resulting values
# Genarate input file for R
#########################################
sub parseResults {

    my($begin, $end) = @_;

    open(RAWOUTPUT, ">".$thisdir."/dat/raw-".$begin."-".$end.".data") ||
	die "Sorry, couldn't open file ".$thisdir."/dat/raw-".$begin."-".$end.".data for writting...\n";
    
    print RAWOUTPUT "Bandwidth Latency Size Window Model Time\n";

    my($B, $L ,$S, $W, $Model, $Time);
    my($counter);
    $counter=1;

   
    open(LOGOUTPUT, "<".$thisdir."/log/trace-file-".($begin)."-".($end).".log")||
	die "Sorry, I coundn't open file ".$thisdir."/log/trace-file-".($it)."-".($it).".log for reading...";

    # Copy execution trace into global trace file
    while(<LOGOUTPUT>)
    {
	my $line = $_;
	chomp $line;
	
	# Find bandwidth parameter
	if($line =~ /Bandwidth/){
	    $B = get_value($line);
	    next;
	}

	# Find latency parameter
	if($line =~ /Latency/){
	    $L = get_value($line);
	    next;
	}

	# Find size parameter
	if($line =~ /Size/){
	    $S = get_value($line);
	    next;
	}

	# Find size parameter
	if($line =~ /Window/){
	    $W = get_value($line);
	    next;
	}

	# Find model parameter
	if($line =~ /Model/){
	    $Model = $line;
	    $Model =~ s/.*:\s+(.*)\s*$/$1/x;
	    next;
	}

	# Find time
	if($line =~ /C1\stime:/){
	    $Time=get_value($line);
	    print RAWOUTPUT $counter++," ",$B," ",$L," ",$S," ",$W," ",$Model," ",$Time,"\n";
	    next;
	}
    }

    close(LOGOUTPUT);    
    close(RAWOUTPUT);
}

#########################################
# Parse numerical value within a line
#########################################
sub get_value {
    my($line)=shift;
    $line=" $line";
    $line =~ s/^
                .*[^\d\.+-eE]+         # anything except something in a number
                (
                   [+-]?\ *          # first, match an optional sign
                   (                 # then match integers or f.p. mantissas:
                      \d+            # start out with a ...
                      (
                          \.\d*      # mantissa of the form a.b or a.
                      )?             # ? takes care of integers of the form a
                      |\.\d+         # mantissa of the form .b
                   )
                   ([eE][+-]?\d+)?  # finally, optionally match an exponent
		 )
                \D*$/$1/x;
    return $line;
}
